--
-- TopRopes
-- Specialization changeable top ropes to load chaff
--
-- @author  Stefan Maurus	
-- @date  04/06/14

-- Copyright © Stefan Maurus, www.stefanmaurus.de

--[[
	<topRopes index="INDEX"/>
]]

TopRopes = {};

function TopRopes.prerequisitesPresent(specializations)
	return true;
end;

function TopRopes:load(xmlFile)
	self.setTopRopesVisibility = SpecializationUtil.callSpecializationsFunction("setTopRopesVisibility");
	self.topRopes = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.topRopes#index"));
	self.topRopesAreVis = true;
	self.topRopesTimer = 0;
	setVisibility(self.topRopes, self.topRopesAreVis);
	self.allowFillFromAir = not self.topRopesAreVis;	
	
	self.fillPlaneYOffset = getXMLFloat(xmlFile, "vehicle.topRopes#fillPlaneYOffset");
	self.heightOffset = 0.03;
	self.topRopesSkinned = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.topRopes#skinned"));
	link(getRootNode(), self.topRopesSkinned);
	
	local i = 0;
	self.ropeJoints = {};
	while true do
		local key = string.format("vehicle.topRopes.joint(%d)", i);
		if not hasXMLProperty(xmlFile, key) then
			break;
		end;
		local index = Utils.indexToObject(self.components, getXMLString(xmlFile, key.."#index"));
		local x, y, z = getTranslation(index);
		table.insert(self.ropeJoints, {index=index, baseX=x, baseY=y, baseZ=z});
		i = i + 1;
	end;
	
	self.scrapFloorActivationIndex = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.scrapFloorActivation#index"));
	self.scrapFloorActivationMin = math.rad(getXMLFloat(xmlFile, "vehicle.scrapFloorActivation#min"));	
	self.scrapFloorActivationMax = math.rad(getXMLFloat(xmlFile, "vehicle.scrapFloorActivation#max"));	
	self.scrapFloorActivationHeightOffset = 0.01;
	
	local i = 0;
	self.scrapFloorActivationJoints = {};
	while true do
		local key = string.format("vehicle.scrapFloorActivation.joint(%d)", i);
		if not hasXMLProperty(xmlFile, key) then
			break;
		end;
		local index = Utils.indexToObject(self.components, getXMLString(xmlFile, key.."#index"));
		table.insert(self.scrapFloorActivationJoints, {index=index});
		i = i + 1;
	end;
end;

function TopRopes:delete()
end;

function TopRopes:readStream(streamId, connection)
	self:setTopRopesVisibility(streamReadBool(streamId), true);
end;

function TopRopes:writeStream(streamId, connection) 
	streamWriteBool(streamId, self.topRopesAreVis); 
end;

function TopRopes:setTopRopesVisibility(bool, noEventSend)	
	self.topRopesAreVis = bool;
	setVisibility(self.topRopes, bool);
	setVisibility(self.topRopesSkinned, bool);
	self.allowFillFromAir = not bool;
	SetTopRopesVisibilityEvent.sendEvent(self, bool, noEventSend);
end;

function TopRopes:mouseEvent(posX, posY, isDown, isUp, button)
end;

function TopRopes:keyEvent(unicode, sym, modifier, isDown)
end;

function TopRopes:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	self:setTopRopesVisibility(Utils.getNoNil(getXMLBool(xmlFile, key.."#topRopes"),true));
	return BaseMission.VEHICLE_LOAD_OK;
end;
 
function TopRopes:getSaveAttributesAndNodes(nodeIdent)
	local attributes = 'topRopes="' ..tostring(self.topRopesAreVis).. '"';
	return attributes, nil;
end;

function TopRopes:update(dt)
	if self:getIsActiveForInput() and self.fillLevel == 0 then
		if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA3) then
			self:setTopRopesVisibility(not self.topRopesAreVis);	
		end;
	end;
	
	if not self.topRopesAreVis and self.isTurnedOn then
		self:setIsTurnedOn(false);
		g_currentMission:showBlinkingWarning(g_i18n:getText("TOPROPES_WARNING"), 3000);
		self.topRopesTimer = 3000;
	end;
	
	if self.topRopesTimer > 0 then
		self.topRopesTimer = math.max(self.topRopesTimer-dt, 0);
	end;
end;

function TopRopes:updateTick(dt)
	if self.actLoad_IntSend > 0.01 or self.tipState > 0 then
		for _, joint in pairs(self.ropeJoints) do
			local height = getFillPlaneHeightAtLocalPos(self.fillVolumes[1].volume, joint.baseX, joint.baseZ);
			setTranslation(joint.index, joint.baseX, math.max(self.fillPlaneYOffset+height+self.heightOffset, joint.baseY), joint.baseZ);
		end;
		
		local isheighter = 0;
		for _, joint in pairs(self.scrapFloorActivationJoints) do
			local x, y, z = getWorldTranslation(joint.index);
			local x1, y1, z1 = worldToLocal(self.fillVolumes[1].volume, x, y, z);
			local height = getFillPlaneHeightAtLocalPos(self.fillVolumes[1].volume, x1, z1);
			if height > (y1-self.scrapFloorActivationHeightOffset) then
				isheighter = 2;
			elseif height < (y1-(self.scrapFloorActivationHeightOffset*2)) then
				if isheighter ~= 2 then
					isheighter = 1;
				end;
			end;
		end;
		
		if isheighter == 2 then
			local x2, _, _ = getRotation(self.scrapFloorActivationIndex);
			setRotation(self.scrapFloorActivationIndex, math.max(math.min(x2+math.rad(0.1), self.scrapFloorActivationMax), self.scrapFloorActivationMin), 0, 0);
		elseif isheighter == 1 then
			local x2, _, _ = getRotation(self.scrapFloorActivationIndex);
			setRotation(self.scrapFloorActivationIndex, math.max(math.min(x2-math.rad(0.5), self.scrapFloorActivationMax), self.scrapFloorActivationMin), 0, 0);
		end;
	end;
end;
	
function TopRopes:draw()
	if self.topRopesTimer > 0 then
	--	g_currentMission:addWarning(g_i18n:getText("TOPROPES_WARNING"));
	end;

	if self.fillLevel == 0 then
		if self.topRopesAreVis then
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("TOPROPES_REMOVE")), InputBinding.IMPLEMENT_EXTRA3);
		else
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("TOPROPES_INSTALL")), InputBinding.IMPLEMENT_EXTRA3);
		end;
	end;
end;

function TopRopes:onAttach()
end;

function TopRopes:onDetach()	
end; 

----Event----

SetTopRopesVisibilityEvent = {};
SetTopRopesVisibilityEvent_mt = Class(SetTopRopesVisibilityEvent, Event);

InitEventClass(SetTopRopesVisibilityEvent, "SetTopRopesVisibilityEvent");

function SetTopRopesVisibilityEvent:emptyNew()
    local self = Event:new(SetTopRopesVisibilityEvent_mt);
    self.className="SetTopRopesVisibilityEvent";
    return self;
end;

function SetTopRopesVisibilityEvent:new(vehicle, topRopesAreVis)
    local self = SetTopRopesVisibilityEvent:emptyNew()
    self.vehicle = vehicle;
	self.topRopesAreVis = topRopesAreVis;
    return self;
end;

function SetTopRopesVisibilityEvent:readStream(streamId, connection)
    local id = streamReadInt32(streamId);
    self.vehicle = networkGetObject(id);

	self.topRopesAreVis = streamReadBool(streamId);
	self.vehicle:setTopRopesVisibility(self.topRopesAreVis, true);
	
	if not connection:getIsServer() then
        g_server:broadcastEvent(SetTopRopesVisibilityEvent:new(self.vehicle, self.topRopesAreVis), nil, connection, self.vehicle);
    end;
end;

function SetTopRopesVisibilityEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.vehicle));
	streamWriteBool(streamId, self.topRopesAreVis);
end;

function SetTopRopesVisibilityEvent.sendEvent(vehicle, topRopesAreVis, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(SetTopRopesVisibilityEvent:new(vehicle, topRopesAreVis), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(SetTopRopesVisibilityEvent:new(vehicle, topRopesAreVis));
		end;
	end;
end;